@@ -1,4 +1,4 @@
-/* $EPIC: crypto.c,v 1.7 2006/09/23 02:56:44 jnelson Exp $ */
+/* $EPIC: crypto.c,v 1.13 2007/09/07 18:07:29 jnelson Exp $ */
 /*
  * crypto.c: SED/CAST5/BLOWFISH/AES encryption and decryption routines.
  *
@@ -165,13 +165,11 @@
 #ifdef HAVE_SSL
 static char *	decipher_evp (const unsigned char *key, int keylen, const unsigned char *ciphertext, int cipherlen, const EVP_CIPHER *type, int *outlen, int ivsize);
 #endif
-static void     decrypt_sed (unsigned char *str, int len, const unsigned char *key, int key_len);
 static char *	decrypt_by_prog (const unsigned char *str, size_t *len, Crypt *key);
 
 #ifdef HAVE_SSL
 static char *	cipher_evp (const unsigned char *key, int keylen, const unsigned char *plaintext, int plaintextlen, const EVP_CIPHER *type, int *retsize, int ivsize);
 #endif
-static void     encrypt_sed (unsigned char *str, int len, char *key, int key_len);
 static char *	encrypt_by_prog (const unsigned char *str, size_t *len, Crypt *key);
 
 unsigned char *	decipher_message (const unsigned char *ciphertext, size_t len, Crypt *key, int *retlen)
@@ -233,7 +231,8 @@ unsigned char *	decipher_message (const unsigned char *ciphertext, size_t len, C
 	    }
 
 	    bytes_to_trim = outbuf[len - 1] & (blocksize - 1);
-	    outbuf[len - bytes_to_trim - 1] = 0;
+	    /* outbuf[len - bytes_to_trim - 1] = 0; */
+	    outbuf[len - bytes_to_trim] = 0; 
 	    memmove(outbuf, outbuf + blocksize, len - blocksize);
 #endif
 	    return outbuf;
@@ -257,7 +256,7 @@ unsigned char *	decipher_message (const unsigned char *ciphertext, size_t len, C
 		return retval;
 	}
 	else
-		panic("decipher_message: key->type %d is not valid", key->type);
+		panic(1, "decipher_message: key->type %d is not valid", key->type);
     }
     while (0);
 
@@ -275,7 +274,7 @@ static char *	decipher_evp (const unsigned char *key, int keylen, const unsigned
 	EVP_CIPHER_CTX_set_padding(&a, 0);
 
 	iv = new_malloc(ivsize);
-	outbuf = new_malloc(1024);
+	outbuf = new_malloc(cipherlen + 1024);
 	memcpy(iv, ciphertext, ivsize);
 
         EVP_DecryptInit_ex(&a, type, NULL, NULL, iv);
@@ -297,7 +296,7 @@ static char *	decipher_evp (const unsigned char *key, int keylen, const unsigned
 }
 #endif
 
-static void     decrypt_sed (unsigned char *str, int len, const unsigned char *key, int key_len)
+void     decrypt_sed (unsigned char *str, int len, const unsigned char *key, int key_len)
 {
         int	key_pos,
                 i;
@@ -411,7 +410,7 @@ unsigned char *	cipher_message (const unsigned char *orig_message, size_t len, C
 		return ciphertext;
 	}
 	else
-		panic("cipher_message: key->type %d is not valid", key->type);
+		panic(1, "cipher_message: key->type %d is not valid", key->type);
 
 	return NULL;
 }
@@ -433,11 +432,11 @@ static char *	cipher_evp (const unsigned char *key, int keylen, const unsigned c
 	iv = new_malloc(ivsize);
 	for (iv_count = 0; iv_count < ivsize; iv_count += sizeof(u_32int_t))
 	{
-		randomval = arc4random();
+		randomval = arc4random();  
 		memmove(iv + iv_count, &randomval, sizeof(u_32int_t));
 	}
 
-	outbuf = new_malloc(1024);
+	outbuf = new_malloc(plaintextlen + 100);
 	memcpy(outbuf, iv, ivsize);
 
         EVP_EncryptInit_ex(&a, type, NULL, NULL, iv);
@@ -457,11 +456,12 @@ static char *	cipher_evp (const unsigned char *key, int keylen, const unsigned c
 	}
 
 	*retsize = outlen + ivsize;
+	new_free(&iv);		/* XXX Is this correct? */
 	return outbuf;
 }
 #endif
 
-static void     encrypt_sed (unsigned char *str, int len, char *key, int key_len)
+void     encrypt_sed (unsigned char *str, int len, const unsigned char *key, int key_len)
 {
         int     key_pos,
                 i;
@@ -512,3 +512,112 @@ static char *	encrypt_by_prog (const unsigned char *str, size_t *len, Crypt *key
         return ret;
 }
 
+/**************************************************************************/
+#ifdef HAVE_SSL
+static void	ext256_key (const char *orig, size_t orig_len, char **key, size_t *keylen)
+{
+	size_t	len;
+
+	if (orig_len < 32)
+		len = orig_len;
+	else
+		len = 32;
+
+	*key = new_malloc(32);
+	memset(*key, 0, 32);
+	memcpy(*key, orig, len);
+	*keylen = 32;
+}
+
+static void	sha256_key (const char *orig, size_t orig_len, char **key, size_t *keylen)
+{
+	*key = new_malloc(32);
+	sha256(orig, orig_len, *key);
+	*keylen = 32;
+}
+
+static void	copy_key (const char *orig, size_t orig_len, char **key, size_t *keylen)
+{
+	*key = malloc_strdup(orig);
+	*keylen = orig_len;
+}
+
+/*
+ * These are helper functions for $xform() to do SSL strong crypto.
+ */
+#define CRYPTO_HELPER_FUNCTIONS(x, y, blocksize, make_key)		\
+ssize_t	x ## _encoder (const char *orig, size_t orig_len, const void *meta, size_t meta_len, char *dest, size_t dest_len) \
+{ 									\
+	size_t	len; 							\
+	int	retsize = 0; 						\
+	char *	retval; 						\
+	char *	realkey;						\
+	size_t	realkeylen;						\
+									\
+	if (orig_len == 0) 						\
+	{ 								\
+		*dest = 0; 						\
+		return 0; 						\
+	} 								\
+									\
+	make_key (meta, meta_len, &realkey, &realkeylen);		\
+	retval = cipher_evp(realkey, realkeylen, orig, orig_len,	\
+				y (), &retsize, blocksize); 		\
+	if (retval && retsize > 0) 					\
+	{ 								\
+		size_t	numb; 						\
+		if (dest_len < (size_t)retsize) 			\
+			numb = dest_len; 				\
+		else 							\
+			numb = retsize; 				\
+		memcpy(dest, retval, numb); 				\
+		dest[numb] = 0; 					\
+		new_free(&retval); 					\
+		return numb; 						\
+	} 								\
+									\
+	if (retval) 							\
+		new_free(&retval); 					\
+	new_free(&realkey);						\
+	return 0; 							\
+}									\
+									\
+ssize_t	x ## _decoder (const char *ciphertext, size_t len, const void *meta, size_t meta_len, char *dest, size_t dest_len) \
+{ 									\
+	unsigned char *	outbuf = NULL; 					\
+	int	bytes_to_trim; 						\
+	int 	retlen = 0; 						\
+	char *	realkey;						\
+	size_t	realkeylen;						\
+									\
+	if (len == 0)							\
+	{								\
+		*dest = 0; 						\
+		return 0; 						\
+	}								\
+									\
+	make_key (meta, meta_len, &realkey, &realkeylen);		\
+	if (!(outbuf = decipher_evp(realkey, realkeylen, ciphertext, len, \
+				y (), &retlen, blocksize))) 		\
+	{ 								\
+		yell("bummer"); 					\
+		return -1; 						\
+	} 								\
+									\
+	bytes_to_trim = outbuf[len - 1] & (blocksize - 1); 		\
+	outbuf[len - bytes_to_trim] = 0;  				\
+	memmove(outbuf, outbuf + blocksize, len - bytes_to_trim); 	\
+									\
+	memcpy(dest, outbuf, retlen); 					\
+	dest[retlen] = 0; 						\
+	new_free(&outbuf);						\
+	return retlen;							\
+}
+
+CRYPTO_HELPER_FUNCTIONS(blowfish, EVP_bf_cbc, 8, copy_key)
+CRYPTO_HELPER_FUNCTIONS(cast5, EVP_cast5_cbc, 8, copy_key)
+CRYPTO_HELPER_FUNCTIONS(aes, EVP_aes_256_cbc, 16, ext256_key)
+CRYPTO_HELPER_FUNCTIONS(aessha, EVP_aes_256_cbc, 16, sha256_key)
+#endif
+
+
